#!/bin/sh

# Load sysconf on systems where the initsystem does not pass the environment
if [ "$1" != "" ]; then
	if [ -r "$1" ]; then
		. "$1"
	else
		echo "Unable to read sysconf from '$1'. Exiting."
		exit 6
	fi
fi

# Set defaults, to overwrite see "@ICINGA2_SYSCONFIGFILE@"

: "${ICINGA2_PID_FILE:="@ICINGA2_FULL_INITRUNDIR@/icinga2.pid"}"
: "${DAEMON:="@CMAKE_INSTALL_FULL_SBINDIR@/icinga2"}"

printf "Removing live host/service objects: "
CONFIG_DIRECTORY="/var/lib/icinga2/api/packages/_api/$(cat /var/lib/icinga2/api/packages/_api/active-stage 2>/dev/null)/conf.d"
if [ -d "${CONFIG_DIRECTORY}" ]; then
	rm -rf "${CONFIG_DIRECTORY}/hosts" >/dev/null 2>&1
	rm -rf "${CONFIG_DIRECTORY}/services" >/dev/null 2>&1
	echo "Done"
else
	echo "Config directory does not exist"
fi

printf "Validating config files: "

OUTPUTFILE="/var/tmp/hlmon/scheduler/validate.out"

if ! "$DAEMON" daemon --validate --color > "$OUTPUTFILE"; then
	echo "Failed"

	cat "$OUTPUTFILE"

	exit 1
fi

echo "Done"

printf "Reloading Icinga 2: "

if [ ! -s "${ICINGA2_PID_FILE}" ]; then
	echo "Error: PID file does not exist"
	exit 7
fi

read MAIN_PID < "${ICINGA2_PID_FILE}"
MAIN_PGID=$(ps h -o pgid "${MAIN_PID}" | sed -e 's/^[[:blank:]]*//' -e 's/[[:blank:]]*$//')
WORKER_PIDS=$(pgrep -g "${MAIN_PGID}" | grep -v "^${MAIN_PGID}$")

if ! kill -HUP "${MAIN_PID}" >/dev/null 2>&1; then
	echo "Error: Icinga not running"
	exit 7
fi

CGROUP_PROCESSES_FILE="/sys/fs/cgroup/systemd/system.slice/icinga2.service/cgroup.procs"
THIS_SCRIPT_PGID=$(ps h -o pgid ${BASHPID} | sed -e 's/^[[:blank:]]*//' -e 's/[[:blank:]]*$//')
if [ -e "${CGROUP_PROCESSES_FILE}" ]; then
	while true; do
		PROCESS_RUNNING="false"
		for WORKER_PID in ${WORKER_PIDS}; do
			if [ -d "/proc/${WORKER_PID}" ]; then
				PROCESS_RUNNING="true"
				break
			fi
		done

		if [[ "${PROCESS_RUNNING}" == "false" && -e "${CGROUP_PROCESSES_FILE}" ]]; then
			ORPHANED_PGIDS=$(ps h -o ppid,pgid $(cat "${CGROUP_PROCESSES_FILE}") | awk '$2 !~ /^('"${MAIN_PID}|${THIS_SCRIPT_PGID}"')$/' | awk '$1 ~ /^1$/' | awk '{print $2}' | sort -u)
			for ORPHANED_PGID in ${ORPHANED_PGIDS}; do
				kill -9 -${ORPHANED_PGID} >/dev/null 2>&1 || :
			done
			break
		fi

		sleep 0.1
	done
fi

echo "Done"
exit 0
